@@ -1,55 +1,52 @@
-import logging
-import tempfile
-import shutil
-import inspect
-import re
+from __future__ import annotations
 
+import functools
+import inspect
+import logging
+import re
+import shutil
+import tempfile
+from collections.abc import Mapping, Sequence
+from datetime import datetime, timedelta
 from io import IOBase
 from pathlib import Path
+from typing import Any, Protocol
+
 from importlib_resources import files
-from datetime import datetime, timedelta
+from importlib_resources.abc import Traversable
 
-from typing import Sequence, Optional, Mapping
-
-from .versions import VersionRaw, guess_version
-from .meta import Meta, MetaRaw, MetaFactory
-from .errors import PinsError
-from .drivers import load_data, save_data, default_title
-from .utils import inform, warn_deprecated, ExtendMethodDoc
+from ._adaptors import Adaptor, create_adaptor
+from .cache import PinsCache
 from .config import get_allow_rsc_short_name
-
+from .drivers import REQUIRES_SINGLE_FILE, default_title, load_data, load_file, save_data
+from .errors import PinsError, PinsVersionError
+from .meta import Meta, MetaFactory, MetaRaw
+from .utils import ExtendMethodDoc, inform, warn_deprecated
+from .versions import VersionRaw, guess_version, version_setup
 
 _log = logging.getLogger(__name__)
 
+_ = default_title  # Keep this import for backward compatibility
 
-# Note that once we drop python 3.7, we can make this a Protocol
-class IFileSystem:
 
-    protocol: "str | list"
+class IFileSystem(Protocol):
+    protocol: str | list
 
-    def ls(self, path: str) -> Sequence[str]:
-        ...
+    def ls(self, path: str) -> Sequence[str]: ...
 
-    def put(self) -> None:
-        ...
+    def put(self) -> None: ...
 
-    def open(self, path: str, mode: str, *args, **kwargs) -> IOBase:
-        ...
+    def open(self, path: str, mode: str, *args, **kwargs) -> IOBase: ...
 
-    def get(self) -> None:
-        ...
+    def get(self) -> None: ...
 
-    def exists(self, path: str, **kwargs) -> bool:
-        ...
+    def exists(self, path: str, **kwargs) -> bool: ...
 
-    def mkdir(self, path, create_parents=True, **kwargs) -> None:
-        ...
+    def mkdir(self, path, create_parents=True, **kwargs) -> None: ...
 
-    def rm(self, path, recursive=False, maxdepth=None) -> None:
-        ...
+    def rm(self, path, recursive=False, maxdepth=None) -> None: ...
 
-    def info(self, path):
-        ...
+    def info(self, path): ...
 
 
 class BaseBoard:
@@ -57,19 +54,15 @@ class BaseBoard:
 
     def __init__(
         self,
-        board: "str | Path",
+        board: str | Path,
         fs: IFileSystem,
         versioned=True,
         meta_factory=MetaFactory(),
-        allow_pickle_read: "bool | None" = None,
+        allow_pickle_read: bool | None = None,
     ):
         self.board = str(board)
         self.fs = fs
         self.meta_factory = meta_factory
-
-        if versioned is False:
-            raise NotImplementedError()
-
         self.versioned = versioned
         self.allow_pickle_read = allow_pickle_read
 
@@ -93,10 +86,15 @@ class BaseBoard:
             Pin name.
 
         """
-        if not self.pin_exists(name):
-            raise PinsError("Cannot check version, since pin %s does not exist" % name)
 
-        versions_raw = self.fs.ls(self.construct_path([self.path_to_pin(name)]))
+        if not self.pin_exists(name):
+            raise PinsError(f"Cannot check version, since pin {name} does not exist")
+
+        detail = isinstance(self, BoardRsConnect)
+
+        versions_raw = self.fs.ls(
+            self.construct_path([self.path_to_pin(name)]), detail=detail
+        )
 
         # get a list of Version(Raw) objects
         all_versions = []
@@ -107,7 +105,6 @@ class BaseBoard:
         # sort them, with latest last
         sorted_versions = self.sort_pin_versions(all_versions)
 
-        # TODO(defer): this deviates from R pins, which returns a df by default
         if as_df:
             import pandas as pd
 
@@ -127,7 +124,7 @@ class BaseBoard:
 
         See Also
         --------
-        BaseBoard.pin_versions
+        [](`~pins.boards.BaseBoard.pin_versions`)
 
         """
 
@@ -139,8 +136,7 @@ class BaseBoard:
             # ensure pin and version exist
             if not self.fs.exists(self.construct_path([pin_name, version])):
                 raise PinsError(
-                    f"Pin {name} either does not exist, "
-                    f"or is missing version: {version}."
+                    f"Pin {name} either does not exist, or is missing version: {version}."
                 )
 
             selected_version = guess_version(version)
@@ -162,9 +158,7 @@ class BaseBoard:
         path_meta = self.construct_path([*components, meta_name])
         f, local = self._open_pin_meta(path_meta)
 
-        meta = self.meta_factory.read_pin_yaml(
-            f, pin_name, selected_version, local=local
-        )
+        meta = self.meta_factory.read_pin_yaml(f, pin_name, selected_version, local=local)
 
         return meta
 
@@ -173,16 +167,16 @@ class BaseBoard:
 
         Notes
         -----
-        This is a low-level function; use pin_search() to get more data about
+        This is a low-level function; use [](`~pins.boards.BaseBoard.pin_search`) to get more data about
         each pin in a convenient form.
         """
 
-        full_paths = self.fs.ls(self.board)
+        full_paths = self.fs.ls(self.board, detail=False)
         pin_names = map(self.keep_final_path_component, full_paths)
 
         return [name for name in pin_names if name not in self.reserved_pin_names]
 
-    def pin_fetch(self, name: str, version: Optional[str] = None) -> Meta:
+    def pin_fetch(self, name: str, version: str | None = None) -> Meta:
         meta = self.pin_meta(name, version)
 
         # TODO: sanity check caching (since R pins does a cache touch here)
@@ -194,7 +188,7 @@ class BaseBoard:
         #       so they could pin_fetch and then examine the result, a la pin_download
         return meta
 
-    def pin_read(self, name, version: Optional[str] = None, hash: Optional[str] = None):
+    def pin_read(self, name, version: str | None = None, hash: str | None = None):
         """Return the data stored in a pin.
 
         Parameters
@@ -205,7 +199,7 @@ class BaseBoard:
             A specific pin version to retrieve.
         hash:
             A hash used to validate the retrieved pin data. If specified, it is
-            compared against the ``pin_hash`` field retrived by ``pin_meta()``.
+            compared against the `pin_hash` field retrieved by [](`~pins.boards.BaseBoard.pin_meta`).
 
         """
         meta = self.pin_fetch(name, version)
@@ -225,68 +219,85 @@ class BaseBoard:
             meta, self.construct_path([pin_name, meta.version.version])
         )
 
-    def pin_write(
+    def _pin_store(
         self,
         x,
-        name: Optional[str] = None,
-        type: Optional[str] = None,
-        title: Optional[str] = None,
-        description: Optional[str] = None,
-        metadata: Optional[Mapping] = None,
-        versioned: Optional[bool] = None,
-        created: Optional[datetime] = None,
+        name: str | None = None,
+        type: str | None = None,
+        title: str | None = None,
+        description: str | None = None,
+        metadata: Mapping | None = None,
+        versioned: bool | None = None,
+        created: datetime | None = None,
+        *,
+        force_identical_write: bool = False,
     ) -> Meta:
-        """Write a pin object to the board.
-
-        Parameters
-        ----------
-        x:
-            An object (e.g. a pandas DataFrame) to pin.
-        name:
-            Pin name.
-        type:
-            File type used to save ``x`` to disk. May be "csv", "arrow", "parquet",
-            "joblib", "json", or "file".
-        title:
-            A title for the pin; most important for shared boards so that others
-            can understand what the pin contains. If omitted, a brief description
-            of the contents will be automatically generated.
-        description:
-            A detailed description of the pin contents.
-        metadata:
-            A dictionary containing additional metadata to store with the pin.
-            This gets stored on the Meta.user field.
-        versioned:
-            Whether the pin should be versioned. Defaults to versioning.
-        created:
-            A date to store in the Meta.created field. This field may be used as
-            part of the pin version name.
-        """
-
-        if type == "feather":
+        _type = type
+        if _type == "feather":
             warn_deprecated(
                 'Writing pin type "feather" is unsupported. Switching type to "arrow".'
                 " This produces the exact same behavior, and also works with R pins."
                 ' Please switch to pin_write using type="arrow".'
             )
-            type = "arrow"
+            _type = "arrow"
+
+        if _type == "file":
+            # the file type makes the name of the data the exact filename, rather
+            # than the pin name + a suffix (e.g. my_pin.csv).
+            if isinstance(x, (tuple, list)) and len(x) == 1:
+                x = x[0]
+
+            if not isinstance(x, (list, tuple)):
+                _p = Path(x)
+                _base_len = len(_p.name) - len("".join(_p.suffixes))
+                object_name = _p.name[:_base_len]
+            else:
+                # multifile upload, keep list of filenames
+                object_name = []
+                for file in x:
+                    _p = Path(file)
+                    # _base_len = len(_p.name) - len("".join(_p.suffixes))
+                    object_name.append(_p.name)  # [:_base_len])
+        else:
+            object_name = None
 
         pin_name = self.path_to_pin(name)
 
+        # Preemptively fetch the most recent pin's meta if it exists - this is used
+        # for the force_identical_write check
+        abort_if_identical = not force_identical_write and self.pin_exists(name)
+        if abort_if_identical:
+            last_meta = self.pin_meta(name)
+
         with tempfile.TemporaryDirectory() as tmp_dir:
-            # create all pin data (e.g. data.txt, save object)
+            # create all pin data (e.g. data.txt, save object) to get the metadata.
+            # For unversioned boards, this also will delete the most recent pin version,
+            # ready for it to be replaced with a new one.
             meta = self.prepare_pin_version(
                 tmp_dir,
                 x,
                 pin_name,
-                type,
+                _type,
                 title,
                 description,
                 metadata,
                 versioned,
                 created,
+                object_name=object_name,
             )
 
+            # force_identical_write check
+            if abort_if_identical:
+                last_hash = last_meta.pin_hash
+
+                if last_hash == meta.pin_hash:
+                    msg = (
+                        f'The hash of pin "{name}" has not changed. Your pin will not '
+                        f"be stored.",
+                    )
+                    inform(log=_log, msg=msg)
+                    return last_meta
+
             # move pin to destination ----
             # create pin version folder
             dst_pin_path = self.construct_path([pin_name])
@@ -311,9 +322,7 @@ class BaseBoard:
                     "but that directory already exists."
                 )
 
-            inform(
-                _log, f"Writing pin:\nName: {repr(pin_name)}\nVersion: {dst_version}"
-            )
+            inform(_log, f"Writing pin:\nName: {repr(pin_name)}\nVersion: {dst_version}")
 
             res = self.fs.put(tmp_dir, dst_version_path, recursive=True)
 
@@ -326,11 +335,80 @@ class BaseBoard:
 
         return meta
 
-    def pin_download(self, name, version=None, hash=None):
-        """TODO: Download the files contained in a pin.
+    def pin_write(
+        self,
+        x,
+        name: str | None = None,
+        type: str | None = None,
+        title: str | None = None,
+        description: str | None = None,
+        metadata: Mapping | None = None,
+        versioned: bool | None = None,
+        created: datetime | None = None,
+        *,
+        force_identical_write: bool = False,
+    ) -> Meta:
+        """Write a pin object to the board.
+
+        Parameters
+        ----------
+        x:
+            An object (e.g. a pandas DataFrame) to pin.
+        name:
+            Pin name.
+        type:
+            File type used to save `x` to disk. May be "csv", "arrow", "parquet",
+            "joblib", or "json".
+        title:
+            A title for the pin; most important for shared boards so that others
+            can understand what the pin contains. If omitted, a brief description
+            of the contents will be automatically generated.
+        description:
+            A detailed description of the pin contents.
+        metadata:
+            A dictionary containing additional metadata to store with the pin.
+            This gets stored on the Meta.user field.
+        versioned:
+            Whether the pin should be versioned. Defaults to versioning.
+        created:
+            A date to store in the Meta.created field. This field may be used as
+            part of the pin version name.
+        force_identical_write:
+            Store the pin even if the pin contents are identical to the last version
+            (compared using the hash). Only the pin contents are compared, not the pin
+            metadata. Defaults to False.
+
+        Returns
+        -------
+        Meta:
+            Metadata about the stored pin. If `force_identical_write` is False and the
+            pin contents are identical to the last version, the last version's metadata
+            is returned.
+        """
+
+        if type == "file":
+            raise NotImplementedError(
+                ".pin_write() does not support type='file'. "
+                "Use .pin_upload() to save a file as a pin."
+            )
+
+        return self._pin_store(
+            x,
+            name,
+            type,
+            title,
+            description,
+            metadata,
+            versioned,
+            created,
+            force_identical_write=force_identical_write,
+        )
+
+    def pin_download(self, name, version=None, hash=None) -> Sequence[str]:
+        """Download the files contained in a pin.
 
         This method only downloads the files in a pin. In order to read and load
-        pin data as an object (e.g. a pandas DataFrame), use ``pin_read()``.
+        pin data as an object (e.g. a pandas DataFrame), use [](`~pins.boards.BaseBoard.pin_read`).
 
         Parameters
         ----------
@@ -340,22 +418,87 @@ class BaseBoard:
             A specific pin version to retrieve.
         hash:
             A hash used to validate the retrieved pin data. If specified, it is
-            compared against the ``pin_hash`` field retrived by ``pin_meta()``.
-
+            compared against the `pin_hash` field retrieved by [](`~pins.boards.BaseBoard.pin_meta`).
 
         """
-        raise NotImplementedError()
 
-    def pin_upload(self, paths, name=None, title=None, description=None, metadata=None):
-        """TODO: Write a pin based on paths to one or more files.
+        meta = self.pin_fetch(name, version)
+
+        if hash is not None:
+            raise NotImplementedError("TODO: validate hash")
+
+        fnames = [meta.file] if isinstance(meta.file, str) else meta.file
+        pin_type = meta.type
+
+        if len(fnames) > 1 and pin_type in REQUIRES_SINGLE_FILE:
+            raise ValueError("Cannot load data when more than 1 file")
+
+        pin_name = self.path_to_pin(name)
+        files = []
+
+        for fname in fnames:
+            # fetch file
+            with load_file(
+                fname,
+                self.fs,
+                self.construct_path([pin_name, meta.version.version]),
+                pin_type,
+            ) as f:
+                # could also check whether f isinstance of PinCache
+                fname = getattr(f, "name", None)
+
+                if fname is None:
+                    raise PinsError("pin_download requires a cache.")
+
+                files.append(str(Path(fname).absolute()))
+
+        return files
+
+    def pin_upload(
+        self,
+        paths: str | list[str],
+        name=None,
+        title=None,
+        description=None,
+        metadata=None,
+    ):
+        """Write a pin based on paths to one or more files.
 
         This method simply uploads the files given, so they can be downloaded later
-        using ``pin_download()``.
+        using [](`~pins.boards.BaseBoard.pin_download`).
+
+        Parameters
+        ----------
+        paths:
+            Paths of files to upload. Currently, only uploading a single file
+            is supported.
+        name:
+            Pin name.
+        title:
+            A title for the pin; most important for shared boards so that others
+            can understand what the pin contains. If omitted, a brief description
+            of the contents will be automatically generated.
+        description:
+            A detailed description of the pin contents.
+        metadata:
+            A dictionary containing additional metadata to store with the pin.
+            This gets stored on the Meta.user field.
         """
-        # TODO(question): why does this method exist? Isn't it equiv to a user
-        # doing this?: pin_write(board, c("filea.txt", "fileb.txt"), type="file")
-        # pin_download makes since, because it will download *regardless of type*
-        raise NotImplementedError()
+
+        if isinstance(paths, (list, tuple)):
+            # check if all paths exist
+            for path in paths:
+                if not Path(path).is_file():
+                    raise PinsError(f"Path is not a valid file: {path}")
+
+        return self._pin_store(
+            paths,
+            name,
+            type="file",
+            title=title,
+            description=description,
+            metadata=metadata,
+        )
 
     def pin_version_delete(self, name: str, version: str):
         """Delete a single version of a pin.
@@ -373,9 +516,7 @@ class BaseBoard:
         pin_version_path = self.construct_path([pin_name, version])
         self.fs.rm(pin_version_path, recursive=True)
 
-    def pin_versions_prune(
-        self, name, n: "int | None" = None, days: "int | None" = None
-    ):
+    def pin_versions_prune(self, name, n: int | None = None, days: int | None = None):
         """Delete old versions of a pin.
 
         Parameters
@@ -383,13 +524,13 @@ class BaseBoard:
         name:
             Pin name.
         n, days:
-            Pick one of n or days to choose how many versions to keep. n = 3 will
-            keep the last three versions, days = 14 will keep all the versions in
+            Pick one of `n` or `days` to choose how many versions to keep. `n = 3` will
+            keep the last three versions; `days = 14` will keep all the versions in
             the last 14 days.
 
         Notes
         -----
-        Regardless of what values you set, pin_versions_prune() will never delete
+        Regardless of what values you set, `pin_versions_prune` will never delete
         the most recent version.
 
         """
@@ -408,7 +549,8 @@ class BaseBoard:
                 raise ValueError("Argument days is {days}, but must be greater than 0.")
 
             date_cutoff = datetime.today() - timedelta(days=days)
-            to_delete = [v for v in versions if v.created < date_cutoff]
+            # Avoid deleting the most recent version
+            to_delete = [v for v in versions[:-1] if v.created < date_cutoff]
 
         # message user about deletions ----
         # TODO(question): how to pin_inform? Log or warning?
@@ -469,7 +611,7 @@ class BaseBoard:
         #               looks with meta objects in it.
         return res
 
-    def pin_delete(self, names: "str | Sequence[str]"):
+    def pin_delete(self, names: str | Sequence[str]):
         """Delete a pin (or pins), removing it from the board.
 
         Parameters
@@ -483,7 +625,7 @@ class BaseBoard:
 
         for name in names:
             if not self.pin_exists(name):
-                raise PinsError("Cannot delete pin, since pin %s does not exist" % name)
+                raise PinsError(f"Cannot delete pin, since pin {name} does not exist")
 
             path_to_pin = self.construct_path([self.path_to_pin(name)])
             self.fs.rm(path_to_pin, recursive=True)
@@ -516,7 +658,7 @@ class BaseBoard:
     def validate_pin_name(self, name: str) -> None:
         """Raise an error if a pin name is not valid."""
 
-        if "/" in name:
+        if name and "/" in name:
             raise ValueError(f"Invalid pin name: {name}")
         elif name in self.reserved_pin_names:
             raise ValueError(f"The pin name '{name}' is reserved for internal use.")
@@ -546,34 +688,70 @@ class BaseBoard:
         self,
         pin_dir_path,
         x,
-        name: Optional[str] = None,
-        type: Optional[str] = None,
-        title: Optional[str] = None,
-        description: Optional[str] = None,
-        metadata: Optional[Mapping] = None,
-        versioned: Optional[bool] = None,
-        created: Optional[datetime] = None,
+        name: str | None = None,
+        type: str | None = None,
+        title: str | None = None,
+        description: str | None = None,
+        metadata: Mapping | None = None,
+        versioned: bool | None = None,
+        created: datetime | None = None,
+        object_name: str | list[str] | None = None,
+    ):
+        meta = self._create_meta(
+            pin_dir_path,
+            x,
+            name,
+            type,
+            title,
+            description,
+            metadata,
+            versioned,
+            created,
+            object_name,
+        )
+
+        # handle unversioned boards
+        version_setup(self, name, meta.version, versioned)
+
+        return meta
+
+    def _create_meta(
+        self,
+        pin_dir_path,
+        x: Adaptor | Any,
+        name: str | None = None,
+        type: str | None = None,
+        title: str | None = None,
+        description: str | None = None,
+        metadata: Mapping | None = None,
+        versioned: bool | None = None,
+        created: datetime | None = None,
+        object_name: str | None = None,
     ):
         if name is None:
             raise NotImplementedError("Name must be specified.")
 
-        if versioned is False:
-            raise NotImplementedError("Only writing versioned pins supported.")
-
         if type is None:
             raise NotImplementedError("Type argument is required.")
 
         if title is None:
-            title = default_title(x, name)
+            title = create_adaptor(x).default_title(name)
 
         # create metadata from object on disk ---------------------------------
         # save all pin data to a temporary folder (including data.txt), so we
         # can fs.put it all straight onto the backend filesystem
-
-        p_obj = Path(pin_dir_path) / name
-
+        apply_suffix = True
+        if isinstance(object_name, (list, tuple)):
+            apply_suffix = False
+            p_obj = []
+            for obj in object_name:
+                p_obj.append(str(Path(pin_dir_path) / obj))
+        elif object_name is None:
+            p_obj = str(Path(pin_dir_path) / name)
+        else:
+            p_obj = str(Path(pin_dir_path) / object_name)
         # file is saved locally in order to hash, calc size
-        file_names = save_data(x, str(p_obj), type)
+        file_names = save_data(x, p_obj, type, apply_suffix)
 
         meta = self.meta_factory.create(
             pin_dir_path,
@@ -632,23 +810,85 @@ class BaseBoard:
 
         # TODO: assumes same_name set to True. Let's require this be set to
         # instantiate a pins cache.
-        if not hasattr(self.fs, "cached_files"):
+        if not isinstance(self.fs, PinsCache):
             return
-
         path_to_hashed = self.fs._check_file(path)
         return touch_access_time(path_to_hashed)
 
 
+def board_deparse(board: BaseBoard):
+    """Return a representation of how a board could be reconstructed.
+
+    Note that this function does not try to represent the exact arguments used
+    to construct a board, but key pieces (like the path to the board). You may
+    need to specify environment variables with API keys to complete the connection.
+
+    Parameters
+    ----------
+    board:
+        A pins board to be represented.
+
+    Examples
+    --------
+
+    The example below deparses a board connected to Posit Connect.
+
+    >>> from pins.constructors import board_connect
+    >>> board_deparse(board_connect(server_url="http://example.com", api_key="xxx"))
+    "board_connect(server_url='http://example.com')"
+
+    Note that the deparsing a Posit Connect board does not keep the api_key,
+    which is sensitive information. In this case, you can set the CONNECT_API_KEY
+    environment variable to connect.
+
+    Below is an example of representing a board connected to a local folder.
+
+    >>> from pins.constructors import board_folder
+    >>> board_deparse(board_folder("a/b/c"))
+    "board_folder('a/b/c')"
+
+    >>> board_deparse(board_folder(path="a/b/c", allow_pickle_read=True))
+    "board_folder('a/b/c', allow_pickle_read=True)"
+    """
+    if board.allow_pickle_read is not None:
+        allow_pickle = f", allow_pickle_read={repr(board.allow_pickle_read)}"
+    else:
+        allow_pickle = ""
+
+    prot = board.fs.protocol
+
+    if prot == "rsc":
+        url = board.fs.api.server_url
+        return f"board_connect(server_url={repr(url)}{allow_pickle})"
+    elif prot in ["file", ("file", "local")]:
+        return f"board_folder({repr(board.board)}{allow_pickle})"
+    elif set(prot) == {"s3", "s3a"}:
+        return f"board_s3({repr(board.board)}{allow_pickle})"
+    elif prot == "abfs":
+        return f"board_azure({repr(board.board)}{allow_pickle})"
+    elif set(prot) == {"gcs", "gs"} or prot == "gs":
+        return f"board_gcs({repr(board.board)}{allow_pickle})"
+    elif prot == "http":
+        return f"board_url({repr(board.board)}, {board.pin_paths}{allow_pickle})"
+    elif prot == "dbc":
+        return f"board_databricks({repr(board.board)}{allow_pickle})"
+    else:
+        raise NotImplementedError(
+            f"board deparsing currently not supported for protocol: {prot}"
+        )
+
+
 class BoardManual(BaseBoard):
-    """Simple board that accepts a dictionary of form pin_name: path.
+    """Simple board that accepts a dictionary of form `pin_name: path`.
 
     Examples
     --------
     >>> import fsspec
     >>> import os
-    >>> fs = fsspec.filesystem("github", org = "machow", repo = "pins-python")
+    >>> fs = fsspec.filesystem("http", block_size=0)
     >>> pin_paths = {"df_csv": "df_csv/20220214T163720Z-9bfad/"}
-    >>> board = BoardManual("pins/tests/pins-compat", fs, pin_paths=pin_paths)
+    >>> url = "https://raw.githubusercontent.com/rstudio/pins-python/main/pins/tests/pins-compat"
+    >>> board = BoardManual(url, fs, pin_paths=pin_paths)
 
     >>> board.pin_list()
     ['df_csv']
@@ -667,15 +907,6 @@ class BoardManual(BaseBoard):
 
         self.pin_paths = pin_paths
 
-    # def pin_read(self, *args, **kwargs):
-    #    if not get_feature_preview():
-    #        raise NotImplementedError(
-    #            "pin_read with BoardManual is currently unimplemented. "
-    #            "See https://github.com/machow/pins-python/issues/59."
-    #        )
-
-    #    return super().pin_read(*args, **kwargs)
-
     @ExtendMethodDoc
     def pin_list(self):
         return list(self.pin_paths)
@@ -716,12 +947,19 @@ class BoardManual(BaseBoard):
         meta = self.pin_meta(name, version)
 
         if isinstance(meta, MetaRaw):
+            f = load_file(meta.file, self.fs, None, meta.type)
+        else:
+            raise NotImplementedError(
+                "TODO: pin_download currently can only read a url to a single file."
+            )
 
-            return self._load_data(meta, None)
+        # could also check whether f isinstance of PinCache
+        fname = getattr(f, "name", None)
 
-        raise NotImplementedError(
-            "TODO: pin_download currently can only read a url to a single file."
-        )
+        if fname is None:
+            raise PinsError("pin_download requires a cache.")
+
+        return [str(Path(fname).absolute())]
 
     def construct_path(self, elements):
         # TODO: in practice every call to construct_path has the first element of
@@ -751,7 +989,6 @@ class BoardManual(BaseBoard):
             return "/".join(pre_components + [pin_path])
         elif len(others) == 2:
             version, meta = others
-
             return "/".join(pre_components + [stripped, meta])
 
         raise NotImplementedError(
@@ -763,14 +1000,14 @@ class BoardRsConnect(BaseBoard):
     # TODO: note that board is unused in this class (e.g. it's not in construct_path())
 
     # TODO: should read template dynamically, not at class def'n time
-    html_assets_dir: Path = files("pins") / "rsconnect/html"
-    html_template: Path = files("pins") / "rsconnect/html/index.html"
+    html_assets_dir: Traversable = files("pins") / "rsconnect/html"
+    html_template: Traversable = files("pins") / "rsconnect/html/index.html"
 
     # defaults work ----
 
     @ExtendMethodDoc
     def pin_list(self):
-        # lists all pin content on RStudio Connect server
+        # lists all pin content on Posit Connect server
         # we can't use fs.ls, because it will list *all content*
         paged_res = self.fs.api.misc_get_applications("content_type:pin")
         results = paged_res.results
@@ -779,7 +1016,7 @@ class BoardRsConnect(BaseBoard):
         return names
 
     @ExtendMethodDoc
-    def pin_write(self, *args, access_type=None, **kwargs):
+    def pin_write(self, *args, access_type=None, versioned: bool | None = None, **kwargs):
         """Write a pin.
 
         Extends parent method in the following ways:
@@ -805,6 +1042,28 @@ class BoardRsConnect(BaseBoard):
                 " can write to it."
             )
 
+        # attempt to make the least number of API calls possible
+        if versioned or versioned is None and self.versioned:
+            # arbitrary number greater than 1
+            n_versions_before = 100
+        else:
+            try:
+                versions_df = self.pin_versions(pin_name, as_df=True)
+                versions = versions_df["version"].to_list()
+                n_versions_before = len(versions)
+            except PinsError:
+                # pin does not exist
+                n_versions_before = 0
+
+        if versioned is None:
+            versioned = True if n_versions_before > 1 else self.versioned
+
+        if versioned is False and n_versions_before > 1:
+            raise PinsVersionError(
+                "Pin is versioned, but you have requested a write without versions."
+                "To un-version a pin, you must delete it"
+            )
+
         # run parent function ---
         meta = f_super(*args, **kwargs)
 
@@ -818,6 +1077,12 @@ class BoardRsConnect(BaseBoard):
             access_type=access_type or content["access_type"],
         )
 
+        # clean up non-active pins in the case of an unversioned board
+        # a pin existed before the latest pin
+        if versioned is False and n_versions_before == 1:
+            _log.info(f"Replacing version '{versions}' with '{meta.version.version}'")
+            self.pin_version_delete(pin_name, versions[0])
+
         return meta
 
     @ExtendMethodDoc
@@ -867,14 +1132,14 @@ class BoardRsConnect(BaseBoard):
             if e.args[0]["code"] != 75:
                 raise e
 
-            raise PinsError("RStudio Connect cannot delete the latest pin version.")
+            raise PinsError("Posit Connect cannot delete the latest pin version.")
 
     @ExtendMethodDoc
     def pin_versions_prune(self, *args, **kwargs):
         sig = inspect.signature(super().pin_versions_prune)
         if sig.bind(*args, **kwargs).arguments.get("days") is not None:
             raise NotImplementedError(
-                "RStudio Connect board cannot prune versions using days."
+                "Posit Connect board cannot prune versions using days."
             )
         super().pin_versions_prune(*args, **kwargs)
 
@@ -901,7 +1166,7 @@ class BoardRsConnect(BaseBoard):
         if not get_allow_rsc_short_name() and name.count("/") != 1:
             raise ValueError(
                 f"Invalid pin name: {name}"
-                "\nRStudio Connect pin names must include user name. E.g. "
+                "\nPosit Connect pin names must include user name. E.g. "
                 "\nsome_user/mtcars, for the user some_user."
             )
 
@@ -933,23 +1198,11 @@ class BoardRsConnect(BaseBoard):
         # to fs.put to the <user>/<content_name>.
         return self.path_to_pin(name)
 
-    @property
+    @functools.cached_property
     def user_name(self):
-        # note that this is essentially the manual version of functools.cached_property
-        # since we support python 3.7
-        name = getattr(self, "_user_name", None)
-        if name is not None:
-            return name
-        else:
-            user = self.fs.api.get_user()
-            self._user_name = user["username"]
-
-            return self._user_name
-
-    def prepare_pin_version(self, pin_dir_path, x, name: "str | None", *args, **kwargs):
-        # TODO: should move board_deparse into utils, to avoid circular import
-        from .constructors import board_deparse
+        return self.fs.api.get_user()["username"]
 
+    def prepare_pin_version(self, pin_dir_path, x, name: str | None, *args, **kwargs):
         # RSC pin names can have form <user_name>/<name>, but this will try to
         # create the object in a directory named <user_name>. So we grab just
         # the <name> part.
@@ -957,7 +1210,7 @@ class BoardRsConnect(BaseBoard):
 
         # TODO(compat): py pins always uses the short name, R pins uses w/e the
         # user passed, but guessing people want the long name?
-        meta = super().prepare_pin_version(pin_dir_path, x, short_name, *args, **kwargs)
+        meta = super()._create_meta(pin_dir_path, x, short_name, *args, **kwargs)
         meta.name = name
 
         # copy in files needed by index.html ----------------------------------
@@ -970,13 +1223,12 @@ class BoardRsConnect(BaseBoard):
             )
 
         # recursively copy all assets into prepped pin version dir
-        # shutil.copytree(self.html_assets_dir, pin_dir_path, dirs_exist_ok=True)
-        _copytree(self.html_assets_dir, pin_dir_path)
+        shutil.copytree(self.html_assets_dir, pin_dir_path, dirs_exist_ok=True)
 
         # render index.html ------------------------------------------------
 
         all_files = [meta.file] if isinstance(meta.file, str) else meta.file
-        pin_files = ", ".join(f"""<a href="{x}">{x}</a>""" for x in all_files)
+        pin_files = ", ".join(f"""<a href="{file}">{file}</a>""" for file in all_files)
 
         context = {
             "date": meta.version.created.replace(microsecond=0),
@@ -984,36 +1236,14 @@ class BoardRsConnect(BaseBoard):
             "pin_files": pin_files,
             "pin_metadata": meta,
             "board_deparse": board_deparse(self),
+            "data_preview": create_adaptor(x).data_preview,
         }
 
-        # data preview ----
-
-        # TODO: move out data_preview logic? Can we draw some limits here?
-        #       note that the R library uses jsonlite::toJSON
-
-        import pandas as pd
-        import json
-
-        if isinstance(x, pd.DataFrame):
-            # TODO(compat) is 100 hard-coded?
-            # Note that we go df -> json -> dict, to take advantage of pandas type conversions
-            data = json.loads(x.head(100).to_json(orient="records"))
-            columns = [
-                {"name": [col], "label": [col], "align": ["left"], "type": [""]}
-                for col in x
-            ]
-
-            # this reproduces R pins behavior, by omitting entries that would be null
-            data_no_nulls = [
-                {k: v for k, v in row.items() if v is not None} for row in data
-            ]
-
-            context["data_preview"] = json.dumps(
-                {"data": data_no_nulls, "columns": columns}
-            )
+        # do not show r code if not round-trip friendly
+        if meta.type in ["joblib"]:
+            context["show_r_style"] = "display:none"
         else:
-            # TODO(compat): set display none in index.html
-            context["data_preview"] = json.dumps({})
+            context["show_r_style"] = ""
 
         # render html template ----
 
@@ -1026,17 +1256,3 @@ class BoardRsConnect(BaseBoard):
         (Path(pin_dir_path) / "index.html").write_text(rendered)
 
         return meta
-
-
-# TODO: replace with shutil.copytree once py3.7 is dropped
-# copied from https://stackoverflow.com/a/12514470/1144523
-def _copytree(src, dst, symlinks=False, ignore=None):
-    import os
-
-    for item in os.listdir(src):
-        s = os.path.join(src, item)
-        d = os.path.join(dst, item)
-        if os.path.isdir(s):
-            shutil.copytree(s, d, symlinks, ignore)
-        else:
-            shutil.copy2(s, d)
