@@ -1,75 +1,26 @@
-import fsspec
+from __future__ import annotations
+
 import os
 import tempfile
+import warnings
+from typing import Callable
 
-from .boards import BaseBoard, BoardRsConnect, BoardManual
-from .cache import PinsCache, PinsRscCache, PinsAccessTimeCache, prefix_cache
-from .config import get_data_dir, get_cache_dir
+import fsspec
+
+from .boards import BaseBoard, BoardManual, BoardRsConnect, board_deparse
+from .cache import PinsAccessTimeCache, PinsCache, PinsRscCacheMapper, prefix_cache
+from .config import get_cache_dir, get_data_dir
+from .errors import PinsError
+
+# Kept here for backward-compatibility reasons
+# Note that this is not a constructor, but a function to represent them.
+board_deparse  # pyright: ignore[reportUnusedExpression]
 
 
 class DEFAULT:
     pass
 
 
-# Representing constructors ===================================================
-
-# Note that this is not a constructor, but a function to represent them.
-def board_deparse(board: BaseBoard):
-    """Return a representation of how a board could be reconstructed.
-
-    Note that this function does not try to represent the exact arguments used
-    to construct a board, but key pieces (like the path to the board). You may
-    need to specify environment variables with API keys to complete the connection.
-
-    Parameters
-    ----------
-    board:
-        A pins board to be represented.
-
-    Examples
-    --------
-
-    The example below deparses a board connected to Posit Connect.
-
-    >>> board_deparse(board_connect(server_url="http://example.com", api_key="xxx"))
-    "board_connect(server_url='http://example.com')"
-
-    Note that the deparsing a Posit Connect board does not keep the api_key,
-    which is sensitive information. In this case, you can set the CONNECT_API_KEY
-    environment variable to connect.
-
-    Below is an example of representing a board connected to a local folder.
-
-    >>> board_deparse(board_folder("a/b/c"))
-    "board_folder('a/b/c')"
-
-    >>> board_deparse(board_folder(path="a/b/c", allow_pickle_read=True))
-    "board_folder('a/b/c', allow_pickle_read=True)"
-    """
-    if board.allow_pickle_read is not None:
-        allow_pickle = f", allow_pickle_read={repr(board.allow_pickle_read)}"
-    else:
-        allow_pickle = ""
-
-    prot = board.fs.protocol
-
-    if prot == "rsc":
-        url = board.fs.api.server_url
-        return f"board_connect(server_url={repr(url)}{allow_pickle})"
-    elif prot == "file":
-        return f"board_folder({repr(board.board)}{allow_pickle})"
-    elif prot == ["s3", "s3a"]:
-        return f"board_s3({repr(board.board)}{allow_pickle})"
-    elif prot == "abfs":
-        return f"board_azure({repr(board.board)}{allow_pickle})"
-    elif prot == ("gcs", "gs"):
-        return f"board_gcs({repr(board.board)}{allow_pickle})"
-    else:
-        raise NotImplementedError(
-            f"board deparsing currently not supported for protocol: {prot}"
-        )
-
-
 # Board constructors ==========================================================
 # note that libraries not used by board classes above are imported within these
 # functions. may be worth moving these funcs into their own module.
@@ -79,41 +30,41 @@ def board(
     protocol: str,
     path: str = "",
     versioned: bool = True,
-    cache: "type[DEFAULT] | None" = DEFAULT,
+    cache: type[DEFAULT] | None = DEFAULT,
     allow_pickle_read=None,
-    storage_options: "dict | None" = None,
-    board_factory: "callable | type[BaseBoard] | None" = None,
+    storage_options: dict | None = None,
+    board_factory: Callable | type[BaseBoard] | None = None,
 ):
     """General function for constructing a pins board.
 
     Note that this is a lower-level function. For most use cases, use a more specific
-    function like board_local(...), or board_s3(...).
+    function like [](`~pins.board_local`), or [](`~pins.board_s3`).
 
     Parameters
     ----------
-    protocol:
+    protocol: str
         File system protocol. E.g. file, s3, github, rsc (for Posit Connect).
-        See fsspec.filesystem for more information.
-    path:
+        See `fsspec.filesystem` for more information.
+    path: str
         A base path the board should use. For example, the directory the board lives in,
-        or the path to its s3 bucket.
-    versioned:
+        or the path to its S3 bucket.
+    versioned: bool
         Whether or not pins should be versioned.
-    cache:
+    cache: optional, type[DEFAULT]
         Whether to use a cache. By default, pins attempts to select the right cache
-        directory, given your filesystem. If None is passed, then no cache will be
-        used. You can set the cache using the PINS_CACHE_DIR environment variable.
+        directory, given your filesystem. If `None` is passed, then no cache will be
+        used. You can set the cache using the `PINS_CACHE_DIR` environment variable.
     allow_pickle_read: optional, bool
         Whether to allow reading pins that use the pickle protocol. Pickles are unsafe,
         and can execute arbitrary code. Only allow reading pickles if you trust the
-        board to be able to execute python code on your computer.
+        board to execute Python code on your computer.
 
-        You can enable reading pickles by setting this to True, or by setting the
-        environment variable PINS_ALLOW_PICKLE_READ. If both are set, this argument
+        You can enable reading pickles by setting this to `True`, or by setting the
+        environment variable `PINS_ALLOW_PICKLE_READ`. If both are set, this argument
         takes precedence.
     storage_options:
         Additional options passed to the underlying filesystem created by
-        fsspec.filesystem.
+        `fsspec.filesystem`.
     board_factory:
         An optional board class to use as the constructor.
 
@@ -121,8 +72,8 @@ def board(
     -----
     Many fsspec implementations of filesystems cache the searching of files, which may
     cause you to not see pins saved by other people. Disable this on these file systems
-    with `storage_options = {"listings_expiry_time": 0}` on s3, or `{"cache_timeout": 0}`
-    on google cloud storage.
+    with `storage_options = {"listings_expiry_time": 0}` on S3, or `{"cache_timeout": 0}`
+    on Google Cloud Storage.
 
     """
 
@@ -137,6 +88,11 @@ def board(
 
         fs = RsConnectFs(**storage_options)
 
+    elif protocol == "dbc":
+        from pins.databricks.fs import DatabricksFs
+
+        fs = DatabricksFs(**storage_options)
+
     else:
         fs = fsspec.filesystem(protocol, **storage_options)
 
@@ -152,8 +108,12 @@ def board(
             board_cache = prefix_cache(fs, hash_prefix)
             cache_dir = os.path.join(base_cache_dir, board_cache)
 
-            fs = PinsRscCache(
-                cache_storage=cache_dir, fs=fs, hash_prefix=hash_prefix, same_names=True
+            fs = PinsCache(
+                cache_storage=cache_dir,
+                fs=fs,
+                hash_prefix=hash_prefix,
+                same_names=True,
+                mapper=PinsRscCacheMapper,
             )
         else:
             # ensures each subdir path is its own cache directory
@@ -185,28 +145,42 @@ def board(
 
 
 def board_folder(path: str, versioned=True, allow_pickle_read=None):
-    """Create a pins board inside a folder.
+    """Use a local folder as a board.
 
     Parameters
     ----------
     path:
         The folder that will hold the board.
-    **kwargs:
-        Passed to the pins.board function.
+    versioned:
+        Whether or not pins should be versioned.
+    allow_pickle_read: optional, bool
+        Whether to allow reading pins that use the pickle protocol. Pickles are unsafe,
+        and can execute arbitrary code. Only allow reading pickles if you trust the
+        board to execute Python code on your computer.
+
+        You can enable reading pickles by setting this to `True`, or by setting the
+        environment variable `PINS_ALLOW_PICKLE_READ`. If both are set, this argument
+        takes precedence.
     """
 
-    return board(
-        "file", path, versioned, cache=None, allow_pickle_read=allow_pickle_read
-    )
+    return board("file", path, versioned, cache=None, allow_pickle_read=allow_pickle_read)
 
 
 def board_temp(versioned=True, allow_pickle_read=None):
-    """Create a pins board in a temporary directory.
+    """Use a local temporary directory as a board.
 
     Parameters
     ----------
-    **kwargs:
-        Passed to the pins.board function.
+    versioned:
+        Whether or not pins should be versioned.
+    allow_pickle_read: optional, bool
+        Whether to allow reading pins that use the pickle protocol. Pickles are unsafe,
+        and can execute arbitrary code. Only allow reading pickles if you trust the
+        board to execute Python code on your computer.
+
+        You can enable reading pickles by setting this to `True`, or by setting the
+        environment variable `PINS_ALLOW_PICKLE_READ`. If both are set, this argument
+        takes precedence.
     """
 
     tmp_dir = tempfile.TemporaryDirectory()
@@ -224,18 +198,24 @@ def board_temp(versioned=True, allow_pickle_read=None):
 
 
 def board_local(versioned=True, allow_pickle_read=None):
-    """Create a board in a system data directory.
+    """Use a local folder as a board.
 
     Parameters
     ----------
-    **kwargs:
-        Passed to the pins.board function.
+    versioned:
+        Whether or not pins should be versioned.
+    allow_pickle_read: optional, bool
+        Whether to allow reading pins that use the pickle protocol. Pickles are unsafe,
+        and can execute arbitrary code. Only allow reading pickles if you trust the
+        board to execute Python code on your computer.
+
+        You can enable reading pickles by setting this to `True`, or by setting the
+        environment variable `PINS_ALLOW_PICKLE_READ`. If both are set, this argument
+        takes precedence.
     """
     path = get_data_dir()
 
-    return board(
-        "file", path, versioned, cache=None, allow_pickle_read=allow_pickle_read
-    )
+    return board("file", path, versioned, cache=None, allow_pickle_read=allow_pickle_read)
 
 
 def board_github(
@@ -247,20 +227,31 @@ def board_github(
     cache=DEFAULT,
     allow_pickle_read=None,
 ):
-    """Returns a github pin board.
+    """Create a board to read and write pins from GitHub.
 
     Parameters
     ----------
     org:
-        Name of the github org (e.g. user account).
+        Name of the GitHub org (e.g. user account).
     repo:
         Name of the repo.
     path:
-        A subfolder in the github repo holding the board.
+        A subfolder in the GitHub repo holding the board.
     token:
-        An optional github token.
-    **kwargs:
-        Passed to the pins.board function.
+        An optional GitHub token.
+    cache:
+        Whether to use a cache. By default, pins attempts to select the right cache
+        directory, given your filesystem. If `None` is passed, then no cache will be
+        used. You can set the cache using the `PINS_CACHE_DIR` environment variable.
+    allow_pickle_read: optional, bool
+        Whether to allow reading pins that use the pickle protocol. Pickles are unsafe,
+        and can execute arbitrary code. Only allow reading pickles if you trust the
+        board to execute Python code on your computer.
+
+        You can enable reading pickles by setting this to `True`, or by setting the
+        environment variable `PINS_ALLOW_PICKLE_READ`. If both are set, this argument
+        takes precedence.
+
 
     Notes
     -----
@@ -307,28 +298,43 @@ def board_urls(*args, **kwargs):
 
 
 def board_url(path: str, pin_paths: dict, cache=DEFAULT, allow_pickle_read=None):
-    """Create a board from individual urls.
+    """Create a board from individual URLs.
 
     Parameters
     ----------
     path:
-        A base url to prefix all individual pin urls with.
+        A base URL to prefix all individual pin URLs with.
     pin_paths: Mapping
-        A dictionary mapping pin name to pin url .
-    **kwargs:
-        Passed to the pins.board function.
+        A dictionary mapping pin name to pin URL.
+    cache:
+        Whether to use a cache. By default, pins attempts to select the right cache
+        directory, given your filesystem. If `None` is passed, then no cache will be
+        used. You can set the cache using the `PINS_CACHE_DIR` environment variable.
+    allow_pickle_read: optional, bool
+        Whether to allow reading pins that use the pickle protocol. Pickles are unsafe,
+        and can execute arbitrary code. Only allow reading pickles if you trust the
+        board to execute Python code on your computer.
 
-    Example
-    -------
+        You can enable reading pickles by setting this to `True`, or by setting the
+        environment variable `PINS_ALLOW_PICKLE_READ`. If both are set, this argument
+        takes precedence.
 
-    >>> github_raw = "https://raw.githubusercontent.com/machow/pins-python/main/pins/tests/pins-compat"
-    >>> pin_paths = {
-    ...     "df_csv": "df_csv/20220214T163720Z-9bfad/",
-    ...     "df_arrow": "df_arrow/20220214T163720Z-ad0c1/",
-    ... }
-    >>> board = board_url(github_raw, pin_paths)
-    >>> board.pin_list()
+
+    Examples
+    --------
+
+    ```python
+    github_raw = "https://raw.githubusercontent.com/rstudio/pins-python/main/pins/tests/pins-compat"
+    pin_paths = {
+      "df_csv": "df_csv/20220214T163720Z-9bfad/",
+      "df_arrow": "df_arrow/20220214T163720Z-ad0c1/",
+      }
+    board = board_url(github_raw, pin_paths)
+    board.pin_list()
+    ```
+    ```
     ['df_csv', 'df_arrow']
+    ```
     """
 
     # TODO(compat): R pins' version is named board_url (no s)
@@ -347,7 +353,7 @@ def board_url(path: str, pin_paths: dict, cache=DEFAULT, allow_pickle_read=None)
     return BoardManual(
         path,
         fs,
-        versioned=True,
+        versioned=False,
         allow_pickle_read=allow_pickle_read,
         pin_paths=pin_paths,
     )
@@ -356,41 +362,66 @@ def board_url(path: str, pin_paths: dict, cache=DEFAULT, allow_pickle_read=None)
 def board_connect(
     server_url=None, versioned=True, api_key=None, cache=DEFAULT, allow_pickle_read=None
 ):
-    """Create a board to read and write pins from an Posit Connect instance.
+    """Create a board to read and write pins from a Posit Connect server.
 
     Parameters
     ----------
     server_url:
-        Url to the Posit Connect server.
+        URL to the Posit Connect server.
+    versioned:
+        Whether or not pins should be versioned.
     api_key:
         API key for server. If not specified, pins will attempt to read it from
-        CONNECT_API_KEY environment variable.
-    **kwargs:
-        Passed to the pins.board function.
+        `CONNECT_API_KEY` environment variable.
+    cache:
+        Whether to use a cache. By default, pins attempts to select the right cache
+        directory, given your filesystem. If `None` is passed, then no cache will be
+        used. You can set the cache using the `PINS_CACHE_DIR` environment variable.
+    allow_pickle_read: optional, bool
+        Whether to allow reading pins that use the pickle protocol. Pickles are unsafe,
+        and can execute arbitrary code. Only allow reading pickles if you trust the
+        board to execute Python code on your computer.
+
+        You can enable reading pickles by setting this to `True`, or by setting the
+        environment variable `PINS_ALLOW_PICKLE_READ`. If both are set, this argument
+        takes precedence.
+
 
     Examples
     --------
-    Use a server url or set the CONNECT_SERVER environt variable to connect:
+    Pins will automatically look for the `CONNECT_SERVER` and `CONNECT_API_KEY` environment variables:
 
-    ::
+    ```python
+    # where environment vars CONNECT_SERVER and CONNECT_API_KEY are set
+    board = board_connect()
+    ```
 
-       server_url = "https://connect.rstudioservices.com"
-       board = board_connect(server_url)
+    Or use the [dotenv](https://saurabh-kumar.com/python-dotenv/) package to load other environment variable names from a `.env` file:
 
-    In order to read a public pin, use board_manual with the public pin url.
+    ```python
+    import os
+    from dotenv import load_dotenv, find_dotenv
+    load_dotenv(find_dotenv())
 
-    ::
+    api_key = os.getenv("MY_API_KEY")
+    server_url = os.getenv("MY_CONNECT_URL")
+    board = board_connect(server_url=server_url, api_key=api_key)
+    ```
 
-       # for a pin at https://connect.rstudioservices.com/content/3004/
-       board = board_url(
-           "https://connect.rstudioservices.com/content",
-           {"my_df": "3004/"}
-       )
-       board.pin_read("my_df")
+    In order to read a public pin, use `board_url()` with the public pin URL:
+
+    ```python
+    # for a pin at https://connect.rstudioservices.com/content/3004/
+    board = board_url(
+      "https://connect.rstudioservices.com/content",
+      {"my_df": "3004/"}
+    )
+    board.pin_read("my_df")
+    ```
 
     See Also
     --------
-    board_url : board for connecting to individual pins, using a url or path.
+    [](`~pins.board_url`) : Board for connecting to individual pins, using a URL or path.
 
     """
 
@@ -401,56 +432,108 @@ def board_connect(
         server_url = os.environ.get("CONNECT_SERVER")
 
     kwargs = dict(server_url=server_url, api_key=api_key)
-    return board(
-        "rsc", None, versioned, cache, allow_pickle_read, storage_options=kwargs
-    )
+    return board("rsc", None, versioned, cache, allow_pickle_read, storage_options=kwargs)
 
 
 board_rsconnect = board_connect
 
 
-def board_s3(path, versioned=True, cache=DEFAULT, allow_pickle_read=None):
+def board_s3(
+    path, versioned=True, cache=DEFAULT, allow_pickle_read=None, **storage_options
+):
     """Create a board to read and write pins from an AWS S3 bucket folder.
 
     Parameters
     ----------
     path:
-        Path of form <bucket_name>/<optional>/<subdirectory>.
-    **kwargs:
-        Passed to the pins.board function.
+        Path of form `<bucket_name>/<optional>/<subdirectory>`.
+    versioned:
+        Whether or not pins should be versioned.
+    cache:
+        Whether to use a cache. By default, pins attempts to select the right cache
+        directory, given your filesystem. If `None` is passed, then no cache will be
+        used. You can set the cache using the `PINS_CACHE_DIR` environment variable.
+    allow_pickle_read: optional, bool
+        Whether to allow reading pins that use the pickle protocol. Pickles are unsafe,
+        and can execute arbitrary code. Only allow reading pickles if you trust the
+        board to execute Python code on your computer.
+
+        You can enable reading pickles by setting this to `True`, or by setting the
+        environment variable `PINS_ALLOW_PICKLE_READ`. If both are set, this argument
+        takes precedence.
+    storage_options:
+        Additional keyword arguments to be passed to the underlying fsspec S3FileSystem.
 
     Notes
     -----
-    The s3 board uses the fsspec library (s3fs) to handle interacting with s3.
-    In order to authenticate, set the AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY,
-    and (optionally) AWS_REGION environment variables.
+    The s3 board uses the fsspec library (s3fs) to handle interacting with AWS S3.
+    In order to authenticate, set the `AWS_ACCESS_KEY_ID`, `AWS_SECRET_ACCESS_KEY`,
+    and (optionally) `AWS_REGION` environment variables. If you are using an
+    s3-compatible storage service that is not from AWS, you can pass in the necessary
+    credentials to the `storage_options` dictionary, such as `endpoint_url`, `key`, and
+    `secret`. We recommend setting these as environment variables. An example using
+    Backblaze B2 would look like:
 
-    See https://github.com/fsspec/s3fs
+    Examples
+    --------
+    >>> import pins
+    >>> board = pins.board_s3(
+    ...     "pins-test",
+    ...     endpoint_url=os.getenv("FSSPEC_S3_ENDPOINT_URL"),
+    ...     key=os.getenv("FSSPEC_S3_KEY"),
+    ...     secret=os.getenv("FSSPEC_S3_SECRET"),
+    ... )
+
+    See <https://github.com/fsspec/s3fs>
 
     """
-    # TODO: user should be able to specify storage options here?
+    # Warn user about the use of non-zero listings_expiry_time
+    listings_expiry_time = storage_options.get("listings_expiry_time", 0)
+    if listings_expiry_time != 0:
+        warning_msg = """
+Non-zero `listings_expiry_time` may lead to unexpected behaviour with cache operations.
+We're not discouraging you from setting it to be a non-zero value,
+but we strongly recommend setting it to 0 for optimal performance.
+"""
+        warnings.warn(warning_msg)
 
-    opts = {"listings_expiry_time": 0}
+    # Set options to pass in. Start with storage options provided by user.
+    opts = {**storage_options}
+    # Set listings_expiry_time based on what's provided by user
+    # or the default value of 0.
+    opts.update({"listings_expiry_time": listings_expiry_time})
     return board("s3", path, versioned, cache, allow_pickle_read, storage_options=opts)
 
 
 def board_gcs(path, versioned=True, cache=DEFAULT, allow_pickle_read=None):
-    """Create a board to read and write pins from an Google Cloud Storage bucket folder.
+    """Create a board to read and write pins from a Google Cloud Storage bucket folder.
 
     Parameters
     ----------
     path:
-        Path of form <bucket_name>/<optional>/<subdirectory>.
-    **kwargs:
-        Passed to the pins.board function.
+        Path of form `<bucket_name>/<optional>/<subdirectory>`.
+    versioned:
+        Whether or not pins should be versioned.
+    cache:
+        Whether to use a cache. By default, pins attempts to select the right cache
+        directory, given your filesystem. If `None` is passed, then no cache will be
+        used. You can set the cache using the `PINS_CACHE_DIR` environment variable.
+    allow_pickle_read: optional, bool
+        Whether to allow reading pins that use the pickle protocol. Pickles are unsafe,
+        and can execute arbitrary code. Only allow reading pickles if you trust the
+        board to execute Python code on your computer.
+
+        You can enable reading pickles by setting this to `True`, or by setting the
+        environment variable `PINS_ALLOW_PICKLE_READ`. If both are set, this argument
+        takes precedence.
 
     Notes
     -----
     The gcs board uses the fsspec library (gcsfs) to handle interacting with
-    google cloud storage. Currently, its default mode of authentication
+    Google Cloud Storage. Currently, its default mode of authentication
     is supported.
 
-    See https://gcsfs.readthedocs.io/en/latest/#credentials
+    See <https://gcsfs.readthedocs.io/#credentials>
     """
 
     # GCSFS uses a different name for listings_expiry_time, and then
@@ -460,14 +543,26 @@ def board_gcs(path, versioned=True, cache=DEFAULT, allow_pickle_read=None):
 
 
 def board_azure(path, versioned=True, cache=DEFAULT, allow_pickle_read=None):
-    """Create a board to read and write pins from an Google Cloud Storage bucket folder.
+    """Create a board to read and write pins from an Azure Datalake Filesystem folder.
 
     Parameters
     ----------
     path:
-        Path of form <bucket_name>/<optional>/<subdirectory>.
-    **kwargs:
-        Passed to the pins.board function.
+        Path of form `<bucket_name>/<optional>/<subdirectory>`.
+    versioned:
+        Whether or not pins should be versioned.
+    cache:
+        Whether to use a cache. By default, pins attempts to select the right cache
+        directory, given your filesystem. If `None` is passed, then no cache will be
+        used. You can set the cache using the `PINS_CACHE_DIR` environment variable.
+    allow_pickle_read: optional, bool
+        Whether to allow reading pins that use the pickle protocol. Pickles are unsafe,
+        and can execute arbitrary code. Only allow reading pickles if you trust the
+        board to execute Python code on your computer.
+
+        You can enable reading pickles by setting this to `True`, or by setting the
+        environment variable `PINS_ALLOW_PICKLE_READ`. If both are set, this argument
+        takes precedence.
 
     Notes
     -----
@@ -475,10 +570,66 @@ def board_azure(path, versioned=True, cache=DEFAULT, allow_pickle_read=None):
     Azure Datalake Filesystem (abfs). Currently, its default mode of authentication
     is supported.
 
-    See https://github.com/fsspec/adlfs
+    See <https://github.com/fsspec/adlfs>
     """
 
     opts = {"use_listings_cache": False}
-    return board(
-        "abfs", path, versioned, cache, allow_pickle_read, storage_options=opts
-    )
+    return board("abfs", path, versioned, cache, allow_pickle_read, storage_options=opts)
+
+
+def board_databricks(path, versioned=True, cache=DEFAULT, allow_pickle_read=None):
+    """Create a board to read and write pins from an Databricks Volume folder.
+
+    Parameters
+    ----------
+    path:
+        The path to the target folder inside Unity Catalog. The path must include the
+        catalog, schema, and volume names, preceded by 'Volumes/', for example:
+        "/Volumes/my-catalog/my-schema/my-volume".
+    versioned:
+        Whether or not pins should be versioned.
+    cache:
+        Whether to use a cache. By default, pins attempts to select the right cache
+        directory, given your filesystem. If `None` is passed, then no cache will be
+        used. You can set the cache using the `PINS_CACHE_DIR` environment variable.
+    allow_pickle_read: optional, bool
+        Whether to allow reading pins that use the pickle protocol. Pickles are unsafe,
+        and can execute arbitrary code. Only allow reading pickles if you trust the
+        board to execute Python code on your computer.
+
+        You can enable reading pickles by setting this to `True`, or by setting the
+        environment variable `PINS_ALLOW_PICKLE_READ`. If both are set, this argument
+        takes precedence.
+
+    Notes
+    -----
+    The Databricks board uses the `databricks-sdk` library to authenticate and interact
+    with the Databricks Volume.
+
+    See <https://docs.databricks.com/aws/en/dev-tools/sdk-python>
+
+
+    Examples
+    --------
+
+    >>> import pytest; pytest.skip()
+
+    >>> import pins
+    >>> from dotenv import load_dotenv
+    >>> load_dotenv() # eg, for a .env file with DATABRICKS_HOST and DATABRICKS_TOKEN set
+    >>> board = pins.board_databricks("/Volumes/examples/my-board/test-volume")
+    >>> board.pin_list()
+    ['df_csv']
+
+    >>> board.pin_read("df_csv")
+       x  y  z
+    0  1  a  3
+    1  2  b  4
+    """
+    try:
+        import databricks.sdk  # noqa: F401
+    except ModuleNotFoundError:
+        raise PinsError(
+            "Install the `databricks-sdk` package for Databricks board support."
+        )
+    return board("dbc", path, versioned, cache, allow_pickle_read)
