@@ -1,28 +1,34 @@
-import pytest
-import tempfile
 import os
+import tempfile
+from pathlib import Path
 
+import pytest
 from importlib_resources import files
 from pytest import mark as m
-from pathlib import Path
-from pins.tests.helpers import BoardBuilder, RscBoardBuilder, Snapshot, rm_env
+
+from pins.tests.helpers import (
+    BoardBuilder,
+    DbcBoardBuilder,
+    RscBoardBuilder,
+    Snapshot,
+    rm_env,
+)
 
 EXAMPLE_REL_PATH = "pins/tests/pins-compat"
 PATH_TO_EXAMPLE_BOARD = files("pins") / "tests/pins-compat"
+PATH_TO_EXAMPLE_BOARD_DBC = "/Volumes/workshops/my-board/my-volume/test"
 PATH_TO_EXAMPLE_VERSION = PATH_TO_EXAMPLE_BOARD / "df_csv/20220214T163720Z-9bfad/"
 EXAMPLE_PIN_NAME = "df_csv"
 
 PATH_TO_MANIFEST_BOARD = files("pins") / "tests/pin-board"
 
-# Based on https://github.com/machow/siuba/blob/main/siuba/tests/helpers.py
-BACKEND_MARKS = ["fs_s3", "fs_file", "fs_gcs", "fs_abfs", "fs_rsc"]
-
 # parameters that can be used more than once per session
 params_safe = [
     pytest.param(lambda: BoardBuilder("file"), id="file", marks=m.fs_file),
     pytest.param(lambda: BoardBuilder("s3"), id="s3", marks=m.fs_s3),
     pytest.param(lambda: BoardBuilder("gcs"), id="gcs", marks=m.fs_gcs),
     pytest.param(lambda: BoardBuilder("abfs"), id="abfs", marks=m.fs_abfs),
+    pytest.param(lambda: DbcBoardBuilder("dbc"), id="dbc", marks=m.fs_dbc),
 ]
 
 # rsc should only be used once, because users are created at docker setup time
@@ -43,8 +49,10 @@ def http_example_board_path():
     # backend = BoardBuilder("s3")
     # yield backend.create_tmp_board(str(PATH_TO_EXAMPLE_BOARD.absolute())).board
     # backend.teardown()
-    # TODO: could putting it in a publically available bucket folder
-    return "https://raw.githubusercontent.com/machow/pins-python/main/pins/tests/pins-compat"
+    # TODO: could putting it in a publicly available bucket folder
+    return (
+        "https://raw.githubusercontent.com/machow/pins-python/main/pins/tests/pins-compat"
+    )
 
 
 @pytest.fixture
@@ -62,15 +70,6 @@ def df():
     return pd.DataFrame({"x": [1, 2, 3], "y": ["a", "b", "c"]})
 
 
-@pytest.fixture
-def tmp_dir2():
-    # fixture for offering a temporary directory
-    # note that pytest has a built-in fixture tmp_dir, but it uses the lib py.path
-    # which recommends using pathlib, etc..
-    with tempfile.TemporaryDirectory() as tmp_dir:
-        yield Path(tmp_dir)
-
-
 @pytest.fixture
 def tmp_cache():
     with rm_env("PINS_CACHE_DIR"):
@@ -89,12 +88,3 @@ def tmp_data_dir():
 
 def pytest_addoption(parser):
     parser.addoption("--snapshot-update", action="store_true")
-
-
-def pytest_configure(config):
-    # TODO: better way to define all marks? Can we iterate over params above?
-    for mark_name in BACKEND_MARKS:
-        fs_name = mark_name.split("_")[-1]
-        config.addinivalue_line(
-            "markers", f"{mark_name}: mark test to only run on {fs_name} filesystem."
-        )
