@@ -1,19 +1,20 @@
-import pytest
-
 import contextlib
-import uuid
-import os
+import functools
 import json
+import os
 import shutil
-
-from tempfile import TemporaryDirectory
-from pathlib import Path
-from functools import wraps
+import uuid
 from datetime import datetime
+from functools import wraps
+from pathlib import Path
+from tempfile import TemporaryDirectory
+
+import pytest
+from fsspec import filesystem
 from importlib_resources import files
 
 from pins.boards import BaseBoard, BoardRsConnect
-from fsspec import filesystem
+from pins.constructors import board_databricks
 
 DEFAULT_CREATION_DATE = datetime(2020, 1, 13, 23, 58, 59)
 
@@ -21,12 +22,15 @@ RSC_SERVER_URL = "http://localhost:3939"
 # TODO: should use pkg_resources for this path?
 RSC_KEYS_FNAME = "pins/tests/rsconnect_api_keys.json"
 
+DATABRICKS_VOLUME = "/Volumes/workshops/my-board/my-volume/test"
+
 BOARD_CONFIG = {
     "file": {"path": ["PINS_TEST_FILE__PATH", None]},
-    "s3": {"path": ["PINS_TEST_S3__PATH", "ci-pins"]},
-    "gcs": {"path": ["PINS_TEST_GCS__PATH", "ci-pins"]},
+    "s3": {"path": ["PINS_TEST_S3__PATH", "pins-test-hadley"]},
+    "gcs": {"path": ["PINS_TEST_GCS__PATH", "pins-python"]},
     "abfs": {"path": ["PINS_TEST_AZURE__PATH", "ci-pins"]},
     "rsc": {"path": ["PINS_TEST_RSC__PATH", RSC_SERVER_URL]},
+    "dbc": {"path": ["PINS_TEST_DBC__PATH", DATABRICKS_VOLUME]},
 }
 
 # TODO: Backend initialization should be independent of helpers, but these
@@ -34,6 +38,44 @@ BOARD_CONFIG = {
 #       putting imports inside rsconnect particulars for now
 
 
+def skip_if_dbc(func):
+    """Decorator to skip test if board protocol is 'dbc'"""
+
+    @functools.wraps(func)
+    def wrapper(*args, **kwargs):
+        import inspect
+
+        board = None
+
+        # Get function signature to map args to parameter names.
+        # We have to do this since parameterized pytest runs passes in
+        # args in different orders
+        sig = inspect.signature(func)
+        bound_args = sig.bind_partial(*args, **kwargs)
+        all_args = {**bound_args.arguments, **kwargs}
+
+        if "board" in all_args:
+            board = all_args["board"]
+        elif "board_with_cache" in all_args:
+            board = all_args["board_with_cache"]
+        else:
+            # Check all arguments for something that looks like a board
+            for arg_value in all_args.values():
+                if hasattr(arg_value, "fs") and hasattr(arg_value.fs, "protocol"):
+                    board = arg_value
+                    break
+
+        if board and (
+            board.fs.protocol == "dbc"
+            or (hasattr(board.fs, "fs") and board.fs.fs.protocol == "dbc")
+        ):
+            pytest.skip("All Databricks tests must be read only")
+
+        return func(*args, **kwargs)
+
+    return wrapper
+
+
 def rsc_from_key(name):
     from pins.rsconnect.api import RsConnectApi
 
@@ -114,7 +156,7 @@ class BoardBuilder:
 
         return value
 
-    def create_tmp_board(self, src_board=None) -> BaseBoard:
+    def create_tmp_board(self, src_board=None, versioned=True) -> BaseBoard:
         if self.fs_name == "gcs":
             opts = {"cache_timeout": 0}
         else:
@@ -136,7 +178,7 @@ class BoardBuilder:
             fs.mkdir(board_name)
 
         self.board_path_registry.append(board_name)
-        return BaseBoard(board_name, fs=fs)
+        return BaseBoard(board_name, fs=fs, versioned=versioned)
 
     def teardown_board(self, board):
         board.fs.rm(board.board, recursive=True)
@@ -168,11 +210,10 @@ class RscBoardBuilder(BoardBuilder):
         self.fs_name = fs_name
         self.path = None
 
-    def create_tmp_board(self, src_board=None):
+    def create_tmp_board(self, src_board=None, versioned=True):
         from pins.rsconnect.fs import PinBundleManifest  # noqa
 
-        board = BoardRsConnect("", rsc_fs_from_key("derek"))
-
+        board = BoardRsConnect("", rsc_fs_from_key("derek"), versioned=versioned)
         if src_board is None:
             return board
 
@@ -204,6 +245,39 @@ class RscBoardBuilder(BoardBuilder):
         self.teardown_board(self.create_tmp_board())
 
 
+class DbcBoardBuilder(BoardBuilder):
+    def __init__(self, fs_name, path=None, *args, **kwargs):
+        self.path = None
+        self.fs_name = fs_name
+        self.current_board = None
+        self.volume = DATABRICKS_VOLUME
+
+    def create_tmp_board(self, src_board=None, versioned=True):
+        # TODO: use temp boards when boards are not read-only
+        # temp_name = str(uuid.uuid4())
+        # board_name = os.path.join(self.volume, temp_name)
+        # db_board = board_databricks(board_name, cache=None)
+        # board = BaseBoard(board_name, fs=db_board.fs, versioned=versioned)
+        # if src_board is not None:
+        #     board.fs.put(src_board, board_name)
+
+        db_board = board_databricks(self.volume, cache=None)
+        board = BaseBoard(self.volume, fs=db_board.fs, versioned=versioned)
+        self.current_board = board
+        return board
+
+    def teardown_board(self, board):
+        pass
+        # TODO: update when board not read-only
+        # board.fs.rm(board.board)
+
+    def teardown(self):
+        pass
+        # TODO: update when board not read-only
+        # board = board_databricks(self.volume)
+        # board.fs.rm(self.current_board.board)
+
+
 # Snapshot ====================================================================
 
 
