@@ -1,22 +1,22 @@
-import fsspec
 import os
 import uuid
+from datetime import datetime, timedelta
+from pathlib import Path
+from time import sleep
+
+import fsspec
 import pandas as pd
 import pytest
 
-from pins.tests.helpers import DEFAULT_CREATION_DATE, rm_env
-from pins.config import PINS_ENV_INSECURE_READ
-from pins.errors import PinsError, PinsInsecureReadError
-from pins.meta import MetaRaw
-
-from datetime import datetime, timedelta
-from time import sleep
-from pathlib import Path
-
 # using pytest cases, so that we can pass in fixtures as parameters
 # TODO: this seems like maybe overkill
 from pytest_cases import fixture, parametrize
 
+from pins.config import PINS_ENV_INSECURE_READ
+from pins.errors import PinsError, PinsInsecureReadError, PinsVersionError
+from pins.meta import MetaRaw
+from pins.tests.helpers import DEFAULT_CREATION_DATE, rm_env, skip_if_dbc
+
 
 @fixture
 def df():
@@ -31,6 +31,33 @@ def board(backend):
     backend.teardown()
 
 
+@fixture
+def board_unversioned(backend):
+    yield backend.create_tmp_board(versioned=False)
+    backend.teardown()
+
+
+@fixture
+def board_with_cache(backend):
+    from pins.constructors import board as board_constructor
+    from pins.constructors import board_rsconnect
+
+    board = backend.create_tmp_board()
+
+    if backend.fs_name == "rsc":
+        # The rsconnect board is special, in that it's slower to set up and tear down,
+        # so our test suite uses multiple rsconnect users in testing its API, and
+        # board behavior. As a result, we need to pass the credentials directly in.
+        server_url, api_key = board.fs.api.server_url, board.fs.api.api_key
+        board_with_cache = board_rsconnect(server_url=server_url, api_key=api_key)
+    else:
+        board_with_cache = board_constructor(backend.fs_name, board.board)
+
+    yield board_with_cache
+
+    backend.teardown()
+
+
 # misc ========================================================================
 
 
@@ -44,28 +71,26 @@ def test_board_validate_pin_name_root(board):
 # pin_write ===================================================================
 
 
+@skip_if_dbc
 def test_board_pin_write_default_title(board):
-
     df = pd.DataFrame({"x": [1, 2, 3], "y": [4, 5, 6]})
     meta = board.pin_write(df, "df_csv", title=None, type="csv")
     assert meta.title == "df_csv: a pinned 3 x 2 DataFrame"
 
 
-def test_board_pin_write_prepare_pin(board, tmp_dir2):
-
+@skip_if_dbc
+def test_board_pin_write_prepare_pin(board, tmp_path: Path):
     df = pd.DataFrame({"x": [1, 2, 3], "y": [4, 5, 6]})
 
-    meta = board.prepare_pin_version(
-        str(tmp_dir2), df, "df_csv", title=None, type="csv"
-    )
+    meta = board.prepare_pin_version(str(tmp_path), df, "df_csv", title=None, type="csv")
     assert meta.file == "df_csv.csv"
-    assert (tmp_dir2 / "data.txt").exists()
-    assert (tmp_dir2 / "df_csv.csv").exists()
-    assert not (tmp_dir2 / "df_csv.csv").is_dir()
+    assert (tmp_path / "data.txt").exists()
+    assert (tmp_path / "df_csv.csv").exists()
+    assert not (tmp_path / "df_csv.csv").is_dir()
 
 
+@skip_if_dbc
 def test_board_pin_write_roundtrip(board):
-
     df = pd.DataFrame({"x": [1, 2, 3], "y": [4, 5, 6]})
 
     assert not board.pin_exists("df_csv")
@@ -78,6 +103,7 @@ def test_board_pin_write_roundtrip(board):
     assert loaded_df.equals(df)
 
 
+@skip_if_dbc
 def test_board_pin_write_type_not_specified_error(board):
     class C:
         pass
@@ -86,6 +112,7 @@ def test_board_pin_write_type_not_specified_error(board):
         board.pin_write(C(), "cool_pin")
 
 
+@skip_if_dbc
 def test_board_pin_write_type_error(board):
     class C:
         pass
@@ -96,6 +123,7 @@ def test_board_pin_write_type_error(board):
     assert "MY_TYPE" in exc_info.value.args[0]
 
 
+@skip_if_dbc
 def test_board_pin_write_feather_deprecated(board):
     df = pd.DataFrame({"x": [1, 2, 3]})
 
@@ -103,7 +131,178 @@ def test_board_pin_write_feather_deprecated(board):
         board.pin_write(df, "cool_pin", type="feather")
 
 
-def test_board_pin_write_rsc_index_html(board, tmp_dir2, snapshot):
+@skip_if_dbc
+def test_board_pin_write_file_raises_error(board, tmp_path):
+    df = pd.DataFrame({"x": [1, 2, 3]})
+
+    path = tmp_path.joinpath("data.csv")
+    df.to_csv(path, index=False)
+
+    # TODO: should this error?
+    with pytest.raises(NotImplementedError):
+        board.pin_write(path, "cool_pin", type="file")
+
+
+@skip_if_dbc
+@pytest.mark.parametrize("force_identical_write", [True, False])
+def test_board_pin_write_force_identical_write_pincount(board, force_identical_write):
+    df = pd.DataFrame({"x": [1, 2, 3]})
+
+    # 1min ago to avoid name collision
+    one_min_ago = datetime.now() - timedelta(minutes=1)
+    board.pin_write(df, "cool_pin", type="csv", created=one_min_ago)
+    board.pin_write(
+        df, "cool_pin", type="csv", force_identical_write=force_identical_write
+    )
+    versions = board.pin_versions("cool_pin")
+    if force_identical_write:
+        assert len(versions) == 2
+    else:
+        assert len(versions) == 1
+
+
+@skip_if_dbc
+def test_board_pin_write_force_identical_write_msg(
+    board, capfd: pytest.CaptureFixture[str]
+):
+    df = pd.DataFrame({"x": [1, 2, 3]})
+
+    # 1min ago to avoid name collision
+    one_min_ago = datetime.now() - timedelta(minutes=1)
+    board.pin_write(df, "cool_pin", type="csv", created=one_min_ago)
+    board.pin_write(df, "cool_pin", type="csv")
+    versions = board.pin_versions("cool_pin")
+
+    _, err = capfd.readouterr()
+    msg = 'The hash of pin "cool_pin" has not changed. Your pin will not be stored.'
+    assert msg in err
+    assert len(versions) == 1
+
+
+@skip_if_dbc
+def test_board_pin_download(board_with_cache, tmp_path):
+    # create and save data
+    df = pd.DataFrame({"x": [1, 2, 3]})
+
+    path = tmp_path / "data.csv"
+    df.to_csv(path, index=False)
+
+    meta = board_with_cache.pin_upload(path, "cool_pin")
+    assert meta.type == "file"
+
+    (pin_path,) = board_with_cache.pin_download("cool_pin")
+    df = pd.read_csv(pin_path)
+    assert df.x.tolist() == [1, 2, 3]
+
+    with pytest.raises(NotImplementedError):
+        board_with_cache.pin_read("cool_pin")
+
+
+@skip_if_dbc
+def test_board_pin_download_filename_many_suffixes(board_with_cache, tmp_path):
+    # create and save data
+    df = pd.DataFrame({"x": [1, 2, 3]})
+
+    path = tmp_path / "data.a.b.csv"
+    df.to_csv(path, index=False)
+
+    board_with_cache.pin_upload(path, "cool_pin")
+
+    (pin_path,) = board_with_cache.pin_download("cool_pin")
+    assert Path(pin_path).name == "data.a.b.csv"
+
+    df = pd.read_csv(pin_path)
+    assert df.x.tolist() == [1, 2, 3]
+
+
+@skip_if_dbc
+def test_board_pin_download_filename_no_suffixes(board_with_cache, tmp_path):
+    # create and save data
+    df = pd.DataFrame({"x": [1, 2, 3]})
+
+    path = tmp_path / "data"
+    df.to_csv(path, index=False)
+
+    board_with_cache.pin_upload(path, "cool_pin")
+
+    (pin_path,) = board_with_cache.pin_download("cool_pin")
+    assert Path(pin_path).name == "data"
+
+    df = pd.read_csv(pin_path)
+    assert df.x.tolist() == [1, 2, 3]
+
+
+@skip_if_dbc
+def test_board_pin_download_filename(board_with_cache, tmp_path):
+    # create and save data
+    df = pd.DataFrame({"x": [1, 2, 3]})
+
+    path = tmp_path / "data.csv"
+    df.to_csv(path, index=False)
+
+    meta = board_with_cache.pin_upload(path, "cool_pin")
+
+    assert meta.file == "data.csv"
+
+    (pin_path,) = board_with_cache.pin_download("cool_pin")
+    assert Path(pin_path).name == "data.csv"
+
+
+@skip_if_dbc
+def test_board_pin_download_no_cache_error(board, tmp_path):
+    df = pd.DataFrame({"x": [1, 2, 3]})
+    path = tmp_path / "data.csv"
+    df.to_csv(path, index=False)
+
+    # TODO: should this error?
+    meta = board.pin_upload(path, "cool_pin")
+    assert meta.type == "file"
+
+    # file boards work okay, since the board directory itself is the cache
+    if board.fs.protocol in ["file", ("file", "local")]:
+        pytest.skip()
+
+    # uncached boards should fail, since nowhere to store the download
+    with pytest.raises(PinsError):
+        (pin_path,) = board.pin_download("cool_pin")
+
+
+@skip_if_dbc
+def test_board_pin_upload_path_list(board_with_cache, tmp_path):
+    # create and save data
+    df = pd.DataFrame({"x": [1, 2, 3]})
+
+    path = tmp_path / "data.csv"
+    df.to_csv(path, index=False)
+
+    meta = board_with_cache.pin_upload([path], "cool_pin")
+    assert meta.type == "file"
+
+    (pin_path,) = board_with_cache.pin_download("cool_pin")
+
+
+@skip_if_dbc
+def test_board_pin_download_filename_multifile(board_with_cache, tmp_path):
+    # create and save data
+    df = pd.DataFrame({"x": [1, 2, 3]})
+
+    path1, path2 = tmp_path / "data1.csv", tmp_path / "data2.csv"
+    df.to_csv(path1, index=False)
+    df.to_csv(path2, index=False)
+
+    meta = board_with_cache.pin_upload([path1, path2], "cool_pin")
+
+    assert meta.type == "file"
+    assert meta.file == ["data1.csv", "data2.csv"]
+
+    pin_path = board_with_cache.pin_download("cool_pin")
+
+    assert len(pin_path) == 2
+    assert Path(pin_path[0]).name == "data1.csv"
+    assert Path(pin_path[1]).name == "data2.csv"
+
+
+def test_board_pin_write_rsc_index_html(board, tmp_path: Path, snapshot):
     if board.fs.protocol != "rsc":
         pytest.skip()
 
@@ -112,7 +311,7 @@ def test_board_pin_write_rsc_index_html(board, tmp_dir2, snapshot):
     pin_name = "test_rsc_pin"
 
     board.prepare_pin_version(
-        str(tmp_dir2),
+        str(tmp_path),
         df,
         pin_name,
         type="csv",
@@ -121,12 +320,13 @@ def test_board_pin_write_rsc_index_html(board, tmp_dir2, snapshot):
         created=DEFAULT_CREATION_DATE,
     )
 
-    snapshot.assert_equal_dir(tmp_dir2)
+    snapshot.assert_equal_dir(tmp_path)
 
 
 # pin_write against different types -------------------------------------------
 
 
+@skip_if_dbc
 @parametrize(
     "obj, type_",
     [
@@ -150,6 +350,7 @@ def test_board_pin_write_type(board, obj, type_, request):
         obj == dst_obj
 
 
+@skip_if_dbc
 def test_board_pin_read_insecure_fail_default(board):
     board.pin_write({"a": 1}, "test_pin", type="joblib", title="some title")
     with pytest.raises(PinsInsecureReadError) as exc_info:
@@ -158,6 +359,7 @@ def test_board_pin_read_insecure_fail_default(board):
     assert "joblib" in exc_info.value.args[0]
 
 
+@skip_if_dbc
 def test_board_pin_read_insecure_fail_board_flag(board):
     # board flag prioritized over env var
     with rm_env(PINS_ENV_INSECURE_READ):
@@ -168,6 +370,7 @@ def test_board_pin_read_insecure_fail_board_flag(board):
             board.pin_read("test_pin")
 
 
+@skip_if_dbc
 def test_board_pin_read_insecure_succeed_board_flag(board):
     # board flag prioritized over env var
     with rm_env(PINS_ENV_INSECURE_READ):
@@ -177,6 +380,66 @@ def test_board_pin_read_insecure_succeed_board_flag(board):
         board.pin_read("test_pin")
 
 
+# pin_write with unversioned boards ===========================================
+
+
+@skip_if_dbc
+@pytest.mark.parametrize("versioned", [None, False])
+def test_board_unversioned_pin_write_unversioned_force_identical_write(
+    versioned, board_unversioned
+):
+    # 1min ago to avoid name collision
+    one_min_ago = datetime.now() - timedelta(minutes=1)
+    board_unversioned.pin_write(
+        {"a": 1},
+        "test_pin",
+        type="json",
+        versioned=versioned,
+        created=one_min_ago,
+        force_identical_write=True,
+    )
+    board_unversioned.pin_write(
+        {"a": 2},
+        "test_pin",
+        type="json",
+        versioned=versioned,
+        force_identical_write=True,
+    )
+
+    assert len(board_unversioned.pin_versions("test_pin")) == 1
+    assert board_unversioned.pin_read("test_pin") == {"a": 2}
+
+
+@skip_if_dbc
+@pytest.mark.parametrize("versioned", [None, False])
+def test_board_unversioned_pin_write_unversioned(versioned, board_unversioned):
+    board_unversioned.pin_write({"a": 1}, "test_pin", type="json", versioned=versioned)
+    board_unversioned.pin_write({"a": 2}, "test_pin", type="json", versioned=versioned)
+
+    assert len(board_unversioned.pin_versions("test_pin")) == 1
+    assert board_unversioned.pin_read("test_pin") == {"a": 2}
+
+
+@skip_if_dbc
+def test_board_unversioned_pin_write_versioned(board_unversioned):
+    board_unversioned.pin_write({"a": 1}, "test_pin", type="json", versioned=False)
+    board_unversioned.pin_write({"a": 2}, "test_pin", type="json", versioned=True)
+
+    assert len(board_unversioned.pin_versions("test_pin")) == 2
+
+
+@skip_if_dbc
+def test_board_versioned_pin_write_unversioned(board):
+    # should fall back to the versioned setting of the board
+    board.pin_write({"a": 1}, "test_pin", type="json")
+    board.pin_write({"a": 2}, "test_pin", type="json")
+
+    with pytest.raises(PinsVersionError):
+        board.pin_write({"a": 3}, "test_pin", type="json", versioned=False)
+
+    assert len(board.pin_versions("test_pin")) == 2
+
+
 # pin_delete ==================================================================
 
 
@@ -187,9 +450,17 @@ def pin_name():
 
 @pytest.fixture
 def pin_del(board, df, pin_name):
-    meta_old = board.pin_write(df, pin_name, type="csv", title="some title")
-    sleep(1)
-    meta_new = board.pin_write(df, pin_name, type="csv", title="some title")
+    # TODO: update when dbc boards no longer read-only
+    if board.fs.protocol == "dbc":
+        pytest.skip()
+    # 1min ago to avoid name collision
+    one_min_ago = datetime.now() - timedelta(minutes=1)
+    meta_old = board.pin_write(
+        df, pin_name, type="csv", title="some title", created=one_min_ago
+    )
+    meta_new = board.pin_write(
+        df, pin_name, type="csv", title="some title", force_identical_write=True
+    )
 
     assert len(board.pin_versions(pin_name)) == 2
     assert meta_old.version.version != meta_new.version.version
@@ -199,13 +470,30 @@ def pin_del(board, df, pin_name):
 
 @pytest.fixture
 def pin_prune(board, df, pin_name):
+    # TODO: update when dbc boards no longer read-only
+    if board.fs.protocol == "dbc":
+        pytest.skip()
     today = datetime.now()
     day_ago = today - timedelta(days=1, minutes=1)
     two_days_ago = today - timedelta(days=2, minutes=1)
 
     board.pin_write(df, pin_name, type="csv", title="some title", created=today)
-    board.pin_write(df, pin_name, type="csv", title="some title", created=day_ago)
-    board.pin_write(df, pin_name, type="csv", title="some title", created=two_days_ago)
+    board.pin_write(
+        df,
+        pin_name,
+        type="csv",
+        title="some title",
+        created=day_ago,
+        force_identical_write=True,
+    )
+    board.pin_write(
+        df,
+        pin_name,
+        type="csv",
+        title="some title",
+        created=two_days_ago,
+        force_identical_write=True,
+    )
 
     versions = board.pin_versions(pin_name, as_df=False)
     assert len(versions) == 3
@@ -250,7 +538,6 @@ def test_board_pin_version_delete_latest(board, pin_name, pin_del):
 
 @pytest.mark.parametrize("n", [1, 2])
 def test_board_pin_versions_prune_n(board, pin_prune, pin_name, n):
-
     board.pin_versions_prune(pin_name, n=n)
     new_versions = board.pin_versions(pin_name, as_df=False)
 
@@ -265,8 +552,7 @@ def test_board_pin_versions_prune_n(board, pin_prune, pin_name, n):
 
 @pytest.mark.parametrize("days", [1, 2])
 def test_board_pin_versions_prune_days(board, pin_prune, pin_name, days):
-
-    # RStudio cannot handle days, since it involves pulling metadata
+    # Posit cannot handle days, since it involves pulling metadata
     if board.fs.protocol == "rsc":
         with pytest.raises(NotImplementedError):
             board.pin_versions_prune(pin_name, days=days)
@@ -280,6 +566,34 @@ def test_board_pin_versions_prune_days(board, pin_prune, pin_name, days):
     assert len(new_versions) == days
 
 
+@skip_if_dbc
+def test_board_pin_versions_prune_days_protect_most_recent(board, pin_name):
+    """To address https://github.com/rstudio/pins-python/issues/297"""
+    # Posit cannot handle days, since it involves pulling metadata
+    if board.fs.protocol == "rsc":
+        with pytest.raises(NotImplementedError):
+            board.pin_versions_prune(pin_name, days=5)
+        return
+
+    today = datetime.now()
+    two_days_ago = today - timedelta(days=2, minutes=1)
+    three_days_ago = today - timedelta(days=3, minutes=1)
+
+    # Note, we are _not_ going to write a pin for today, otherwise we wouldn't be
+    # properly testing the protection of the most recent version - it would be trivially
+    # protected because it would always lie in the last day / fraction of a day.
+    board.pin_write({"a": 1}, pin_name, type="json", created=two_days_ago)
+    assert len(board.pin_versions(pin_name, as_df=False)) == 1
+    board.pin_write({"a": 2}, pin_name, type="json", created=three_days_ago)
+
+    # prune the versions, keeping only the most recent
+    board.pin_versions_prune(pin_name, days=1)
+
+    # check that only the most recent version remains
+    versions = board.pin_versions(pin_name, as_df=False)
+    assert len(versions) == 1
+
+
 # pin_search ==================================================================
 
 
@@ -296,6 +610,7 @@ def test_board_pin_versions_prune_days(board, pin_prune, pin_name, days):
         ("the-title", ["x-pin-1", "x-pin-2", "y-pin-1", "y-z"]),
     ],
 )
+@skip_if_dbc
 def test_board_pin_search_name(board, df, search, matches):
     if board.fs.protocol == "rsc":
         matches = ["derek/" + m for m in matches]
@@ -318,15 +633,14 @@ from pins.boards import BaseBoard  # noqa
 from pins.cache import PinsCache  # noqa
 
 
-def test_board_base_pin_meta_cache_touch(tmp_dir2, df):
-
+def test_board_base_pin_meta_cache_touch(tmp_path: Path, df):
     cache = fsspec.filesystem(
         "pinscache",
         target_protocol="file",
         same_names=True,
-        hash_prefix=str(tmp_dir2),
+        hash_prefix=str(tmp_path),
     )
-    board = BaseBoard(str(tmp_dir2), fs=cache)
+    board = BaseBoard(str(tmp_path), fs=cache)
 
     board.pin_write(df, "some_df", type="csv")
     meta = board.pin_meta("some_df")
@@ -346,7 +660,7 @@ def test_board_base_pin_meta_cache_touch(tmp_dir2, df):
     assert orig_access < new_access
 
 
-# RStudio Connect specific ====================================================
+# Posit Connect specific ====================================================
 
 # import fixture that builds / tearsdown user "susan"
 from pins.tests.test_rsconnect_api import (  # noqa
@@ -390,7 +704,9 @@ def test_board_pin_search_admin_user(df, board_short, fs_admin):  # noqa
 @pytest.mark.fs_rsc
 def test_board_rsc_pin_write_title_update(df, board_short):
     board_short.pin_write(df, "susan/some_df", type="csv", title="title a")
-    board_short.pin_write(df, "susan/some_df", type="csv", title="title b")
+    board_short.pin_write(
+        df, "susan/some_df", type="csv", title="title b", force_identical_write=True
+    )
 
     content = board_short.fs.info("susan/some_df")
     assert content["title"] == "title b"
@@ -472,9 +788,7 @@ from pins.boards import BoardManual  # noqa
 
 
 def test_board_manual_http_file_download():
-    # TODO: change when repo is moved to RStudio
-
-    path = "https://raw.githubusercontent.com/machow/pins-python"
+    path = "https://raw.githubusercontent.com/rstudio/pins-python"
     license_path = "main/LICENSE"
 
     # use a simple cache, which automatically creates a temporary directory
@@ -502,7 +816,7 @@ def test_board_manual_pin_read():
     # see https://github.com/fsspec/filesystem_spec/issues/389
     fs = fsspec.filesystem("http", block_size=0)
     board = BoardManual(
-        "https://raw.githubusercontent.com/machow/pins-python/main/pins/tests/pins-compat",
+        "https://raw.githubusercontent.com/rstudio/pins-python/main/pins/tests/pins-compat",
         fs,
         pin_paths={
             "df_csv": "df_csv/20220214T163718Z-eceac/",
