@@ -1,17 +1,17 @@
 import os
+from pathlib import Path
+
 import pandas as pd
 import pytest
-
 from pandas.testing import assert_frame_equal
-from pathlib import Path
 
 from pins import constructors as c
 from pins.tests.conftest import (
+    EXAMPLE_REL_PATH,
     PATH_TO_EXAMPLE_BOARD,
     PATH_TO_EXAMPLE_VERSION,
-    EXAMPLE_REL_PATH,
 )
-from pins.tests.helpers import rm_env
+from pins.tests.helpers import rm_env, skip_if_dbc
 
 
 @pytest.fixture
@@ -39,14 +39,18 @@ def construct_from_board(board):
     prot = board.fs.protocol
     fs_name = prot if isinstance(prot, str) else prot[0]
 
-    if fs_name == "file":
+    if fs_name in ["file", ("file", "local")]:
         board = c.board_folder(board.board)
+    elif fs_name == "dbc":
+        board = c.board_databricks(board.board)
     elif fs_name == "rsc":
         board = c.board_rsconnect(
             server_url=board.fs.api.server_url, api_key=board.fs.api.api_key
         )
     elif fs_name == "abfs":
         board = c.board_azure(board.board)
+    elif fs_name == "gs":
+        board = c.board_gcs(board.board)
     else:
         board = getattr(c, f"board_{fs_name}")(board.board)
 
@@ -55,11 +59,12 @@ def construct_from_board(board):
 
 # End-to-end constructor tests
 
+
 # there are two facets of boards: reading and writing.
 # copied from test_compat
 @pytest.mark.skip_on_github
 def test_constructor_board_url_data(tmp_cache, http_example_board_path, df_csv):
-    board = c.board_urls(
+    board = c.board_url(
         http_example_board_path,
         # could derive from example version path
         pin_paths={"df_csv": "df_csv/20220214T163720Z-9bfad/"},
@@ -73,11 +78,13 @@ def test_constructor_board_url_data(tmp_cache, http_example_board_path, df_csv):
 
 @pytest.mark.xfail
 @pytest.mark.skip_on_github
-def test_constructor_board_url_cache(tmp_cache, http_example_board_path, df_csv):
+def test_constructor_board_url_cache(
+    tmp_cache, http_example_board_path, df_csv, tmp_path
+):
     # TODO: downloading a pin does not put files in the same directory, since
     # in this case we are hashing on the full url.
 
-    board = c.board_urls(
+    board = c.board_url(
         http_example_board_path,
         # could derive from example version path
         pin_paths={"df_csv": "df_csv/20220214T163718Z-eceac/"},
@@ -85,6 +92,20 @@ def test_constructor_board_url_cache(tmp_cache, http_example_board_path, df_csv)
 
     board.pin_read("df_csv")
 
+    # cannot write or view pin versions
+
+    with pytest.raises(NotImplementedError):
+        board.pin_write(df_csv)
+    with pytest.raises(NotImplementedError):
+        board.pin_versions("df_csv")
+    with pytest.raises(NotImplementedError):
+        board.pin_version_delete(name="df_csv", version="20220214T163718Z")
+    with pytest.raises(NotImplementedError):
+        df = pd.DataFrame({"x": [1, 2, 3]})
+        path = tmp_path / "data.csv"
+        df.to_csv(path, index=False)
+        board.pin_upload(path, "cool_pin")
+
     # check cache ----
     http_dirs = list(tmp_cache.glob("http_*"))
 
@@ -107,7 +128,7 @@ def test_constructor_board_url_file(tmp_cache, http_example_board_path):
     # TODO: downloading a pin does not put files in the same directory, since
     # in this case we are hashing on the full url.
 
-    board = c.board_urls(
+    board = c.board_url(
         http_example_board_path,
         # could derive from example version path
         pin_paths={"df_csv": "df_csv/20220214T163718Z-eceac/df_csv.csv"},
@@ -127,10 +148,13 @@ def test_constructor_board_url_file(tmp_cache, http_example_board_path):
 
     assert str(res[0]).endswith("df_csv.csv")
 
+    new_board = eval(c.board_deparse(board), c.__dict__)
+    assert new_board.pin_list() == board.pin_list()
+
 
 @pytest.mark.skip_on_github
 def test_constructor_board_github(tmp_cache, http_example_board_path, df_csv):
-    board = c.board_github("machow", "pins-python", EXAMPLE_REL_PATH)  # noqa
+    board = c.board_github("rstudio", "pins-python", EXAMPLE_REL_PATH)  # noqa
 
     df = board.pin_read("df_csv")
     assert_frame_equal(df, df_csv)
@@ -154,6 +178,7 @@ def board(backend):
     backend.teardown_board(board)
 
 
+@skip_if_dbc  # passes, but skipping since this cannot clean itself up properly
 def test_constructor_boards(board, df_csv, tmp_cache):
     # TODO: would be nice to have fixtures for each board constructor
     # doesn't need to copy over pins-compat content
@@ -166,12 +191,16 @@ def test_constructor_boards(board, df_csv, tmp_cache):
     df = board.pin_read("df_csv")
 
     # check data
-    assert_frame_equal(df, df_csv)
+    # TODO: update when dbc boards are not read-only
+    if board.fs.protocol == "dbc":
+        pass
+    else:
+        assert_frame_equal(df, df_csv)
 
     # check the cache structure -----------------------------------------------
 
     # check cache
-    if board.fs.protocol == "file":
+    if board.fs.protocol in ["file", ("file", "local")]:
         # no caching for local file boards
         pass
     else:
@@ -208,6 +237,7 @@ def board2(backend):
     backend.teardown_board(board2)
 
 
+@skip_if_dbc
 def test_constructor_boards_multi_user(board2, df_csv, tmp_cache):
     prot = board2.fs.protocol
     fs_name = prot if isinstance(prot, str) else prot[0]
@@ -235,7 +265,6 @@ def test_constructor_boards_multi_user(board2, df_csv, tmp_cache):
 
 @pytest.mark.skip_on_github
 def test_board_constructor_local_default_writable():
-
     with rm_env("PINS_DATA_DIR"):
         board = c.board_local()
         p_board = Path(board.board)
@@ -253,11 +282,11 @@ def test_board_constructor_temp_writable():
         assert len(list(p_board.glob("*"))) == 0
 
 
-def test_board_constructor_folder(tmp_dir2, df):
-    board = c.board_folder(str(tmp_dir2))
+def test_board_constructor_folder(tmp_path: Path, df):
+    board = c.board_folder(str(tmp_path))
     board.pin_write(df, "some_df", type="csv")
 
-    assert (tmp_dir2 / "some_df").exists()
+    assert (tmp_path / "some_df").exists()
     df2 = board.pin_read("some_df")
 
     assert df.equals(df2)
