@@ -1,5 +1,6 @@
+import hashlib
+import os
 import sys
-
 from functools import update_wrapper
 from types import MethodType
 from warnings import warn
@@ -19,6 +20,14 @@ def warn_deprecated(msg):
     warn(msg, DeprecationWarning)
 
 
+def hash_name(path, same_name):
+    if same_name:
+        _hash = os.path.basename(path)
+    else:
+        _hash = hashlib.sha256(path.encode()).hexdigest()
+    return _hash
+
+
 class ExtendMethodDoc:
     # Note that the indentation assumes these are top-level method docstrings,
     # so are indented 8 spaces (after the initial sentence).
@@ -68,3 +77,11 @@ class ExtendMethodDoc:
         # which allows all the inspect machinery to give sphinx the __call__
         # attribute we set in __init__.
         raise NotImplementedError()
+
+
+# based off fsspec.isfilelike
+def isfilelike(file) -> bool:
+    for attr in ["read", "close", "tell"]:
+        if not hasattr(file, attr):
+            return False
+    return True
