@@ -1,8 +1,9 @@
+from pathlib import Path
+
 import pandas as pd
 
-from pathlib import Path
-from pins.rsconnect.fs import PinBundleManifest
 from pins.meta import MetaFactory
+from pins.rsconnect.fs import PinBundleManifest
 
 p_root = Path("pins/tests/example-bundle")
 p_root.parent.mkdir(parents=True, exist_ok=True)
@@ -15,9 +16,7 @@ df = pd.DataFrame({"x": [1, 2, 3]})
 df.to_csv(p_data)
 
 p_meta = p_root / "data.txt"
-meta = MetaFactory().create(
-    str(p_data), "csv", title="some title", name="data_frame.csv"
-)
+meta = MetaFactory().create(str(p_data), "csv", title="some title", name="data_frame.csv")
 meta.to_yaml(p_meta.open("w"))
 
 # add manifest last, since it enumerates all the files
